@@ -19,17 +19,45 @@
 # Place - Suite 330, Boston, MA 02111-1307, USA.
 #
 
+umask 077
+
+USING_ZSH=
+USING_BASH=
+NEED_POSIX=
+
 # For zsh sanity...
 #   allows splitting strings on whitespace in zsh.
-setopt SH_WORD_SPLIT 2>/dev/null || true
+setopt SH_WORD_SPLIT 2>/dev/null && USING_ZSH=1 || true
 #   allows sourced files to know they're sourced in zsh.
 unsetopt FUNCTION_ARGZERO 2>/dev/null || true
 
+# Detect bash and tell it not to be POSIX, because we like it better that way.
+shopt > /dev/null 2>&1 && USING_BASH=1
+[ -n "$USING_BASH" ] && set +o posix 2>/dev/null
+
+[ -z "$USING_BASH$USING_ZSH" ] && NEED_POSIX=1
+
+# We prefer to use dash if we've got it.
+if [ -z "$NEED_POSIX" ] && [ -z "$TRIED_DASH" ] &&
+	command -v dash > /dev/null 2>&1 ; then
+
+    if [ "`basename $0`" = "$0" ] ; then
+	myself=`command -v $0` || myself=
+    elif [ -x "$0" ] ; then
+	myself=$0
+    fi
+
+    TRIED_DASH=1
+    export TRIED_DASH
+    [ -n "$myself" ] && exec dash $myself $*
+fi
+unset TRIED_DASH
+
 SWSUSP_D="/etc/hibernate"
 SCRIPTLET_PATH="$SWSUSP_D/scriptlets.d /usr/local/share/hibernate/scriptlets.d /usr/share/hibernate/scriptlets.d"
 CONFIG_FILE="$SWSUSP_D/hibernate.conf"
 EXE=`basename $0`
-VERSION="1.02"
+VERSION="1.06"
 
 # Add these to the $PATH just in case.
 PATH="$PATH:/sbin:/usr/sbin"
@@ -153,6 +181,51 @@ $WRAPPED_HELP
 }
 CONFIGURATION_OPTIONS_HELP=""
 
+FindXServer() {
+    [ -n "$FIND_X_SERVER_RESULT" ] && return $FIND_X_SERVER_RESULT
+
+    [ -z "$DISPLAY" ] && DISPLAY=:0
+    export DISPLAY
+
+    # See if we need to authenticate to this X server.
+    PATH=$PATH:/usr/bin/X11:/usr/X11R6/bin
+    export PATH
+    xhost=`command -v xhost 2>/dev/null`
+    if [ $? -ne 0 ] ; then
+	vecho 0 "$EXE: Could not find xhost program. Disabling X scriptlets."
+	FIND_X_SERVER_RESULT=1
+	return 1
+    fi
+
+    # Find a useful XAUTHORITY and ideally a username too if we can!
+    local xuser xauth xpid
+    for xpid in `pidof kwrapper ksmserver kdeinit gnome-session X XFree86 Xorg` ; do
+	xauth=`awk 'BEGIN{RS="\\000";FS="="}($1 == "XAUTHORITY"){print $2}' < /proc/$xpid/environ`
+	xhome=`awk 'BEGIN{RS="\\000";FS="="}($1 == "HOME"){print $2}' < /proc/$xpid/environ`
+	xuser=`/bin/ls -ld /proc/$xpid/ | awk '{print $3}'`
+	[ -z $xauth ] && [ -n $xhome ] && [ -f $xhome/.Xauthority ] && xauth=$xhome/.Xauthority
+
+	XAUTHORITY=$xauth su $xuser -c "$xhost" > /dev/null 2>&1 && break
+
+	xauth=
+	xuser=
+    done
+    if [ -n $xuser ] ; then
+	XUSER=$xuser
+	if [ -n "$xauth" ] ; then
+	    XAUTHORITY=$xauth
+	    export XAUTHORITY
+	fi
+    else
+	FIND_X_SERVER_RESULT=1
+	vecho 0 "$EXE: Could not attach to X server on $DISPLAY. Disabling X scriptlets."
+	return 1
+    fi
+
+    FIND_X_SERVER_RESULT=0
+    return 0
+}
+
 ##############################################################################
 ### Helper functions                                                       ###
 ##############################################################################
@@ -239,13 +312,13 @@ PluginConfigOption() {
 EnsureHavePrerequisites() {
     local i
     for i in awk grep sort getopt basename ; do
-	if ! which $i > /dev/null; then
-	    echo "Could not find required program \"$i\". Aborting."
+	if ! command -v $i > /dev/null; then
+	    vecho 0 "Could not find required program \"$i\". Aborting."
 	    exit 1
 	fi
     done
     # Improvise printf using awk if need be.
-    if ! which printf > /dev/null 2>&1 ; then
+    if ! command -v printf > /dev/null 2>&1 ; then
 	# This implementation fails on strings that contain double quotes.
 	# It does the job for the help screen at least.
 	printf() {
@@ -261,7 +334,7 @@ EnsureHavePrerequisites() {
 	}
     fi
     # Improvise mktemp in case we need it too!
-    if ! which mktemp > /dev/null 2>&1 ; then
+    if ! command -v mktemp > /dev/null 2>&1 ; then
 	# Use a relatively safe equivalent of mktemp. Still suspectible to race
 	# conditions, but highly unlikely.
 	mktemp() {
@@ -271,7 +344,7 @@ EnsureHavePrerequisites() {
 	    CNT=1
 	    while :; do
 		D=`date +%s`
-		FN=/tmp/swsusptemp-$$$D$RANDOM$RANDOM$CNT
+		FN=/tmp/tmp.hibernate.$$$D$RANDOM$RANDOM$CNT
 		[ -f $FN ] && continue
 		touch $FN && break
 		CNT=$(($CNT+1))
@@ -332,7 +405,7 @@ DoGetOpt() {
 	    -v|--verbosity)
 		OPT_VERBOSITY="${1#\'}"
 		OPT_VERBOSITY="${OPT_VERBOSITY%\'}"
-		VERBOSITY="$OPT_VERBOSITY"
+		EnsureNumeric "verbosity" $OPT_VERBOSITY && VERBOSITY="$OPT_VERBOSITY"
 		shift
 		;;
 	    -q)
@@ -467,7 +540,18 @@ BoolIsOn() {
     exit 1
 }
 
-# ProcessConfigOption: takes a configuration option and it's parameters and
+IsANumber() {
+    case "$1" in *[!0-9]*|"") return 1 ;; esac
+    return 0
+}
+
+EnsureNumeric() {
+    IsANumber "$2" && return 0
+    echo "$EXE: Invalid numeric value ($2) for option $1 in configuration file"
+    exit 1
+}
+
+# ProcessConfigOption: takes a configuration option and its parameters and
 # passes it out to the relevant scriptlet.
 ProcessConfigOption() {
     local option
@@ -489,19 +573,30 @@ ProcessConfigOption() {
 		LOGFILE="$params"
 	    ;;
 	logverbosity)
-	    LOG_VERBOSITY="$params"
+	    EnsureNumeric "$option" "$params" && LOG_VERBOSITY="$params"
 	    ;;
 	swsuspvt)
-	    SWSUSPVT="$params"
+	    EnsureNumeric "$option" "$params" && SWSUSPVT="$params"
 	    ;;
 	verbosity)
-	    [ -z "$OPT_VERBOSITY" ] &&
+	    [ -z "$OPT_VERBOSITY" ] && EnsureNumeric "$option" "$params" &&
 		VERBOSITY="$params"
 	    ;;
 	distribution)
-	    [ -z "$DISTRIBUTION" ] &&
-		DISTRIBUTION="$params"
+	    [ -z "$DISTRIBUTION" ] && DISTRIBUTION="$params"
 	    ;;
+	xdisplay)
+	    DISPLAY="$params"
+	    export DISPLAY
+	    ;;
+        include)
+            if [ -r "$params" ]; then
+                ReadConfigFile "$params"
+            else
+                echo "$EXE: Unable to read configuration file $params (from Include directive)."
+                exit 1
+            fi
+            ;;
 	*)
 	    if ! PluginConfigOption $option $params ; then
 		echo "$EXE: Unknown configuration option ($option)"
@@ -516,8 +611,9 @@ ProcessConfigOption() {
 # appropriate variables in the script. Returns 0 on success, exits on errors
 ReadConfigFile() {
     local option params
-    if [ ! -f "$CONFIG_FILE" ] ; then
-	echo "WARNING: No configuration file found ($CONFIG_FILE)."
+    local file_name="$1"
+    if [ ! -f "${file_name}" ] ; then
+	echo "WARNING: No configuration file found (${file_name})."
 	echo "This script probably won't do anything."
 	return 0
     fi
@@ -529,7 +625,7 @@ ReadConfigFile() {
 	[ -z "$option" ] && continue
 	case $option in ""|\#*) continue ;; esac # avoids a function call (big speed hit)
 	ProcessConfigOption $option $params
-    done < $CONFIG_FILE
+    done < ${file_name}
     return 0
 }
 
@@ -555,6 +651,8 @@ AddInbuiltHelp() {
     AddConfigHelp "AlwaysForce <boolean>" "If set to yes, the script will always run as if --force had been passed."
     AddConfigHelp "AlwaysKill <boolean>" "If set to yes, the script will always run as if --kill had been passed."
     AddConfigHelp "Distribution <debian|fedora|mandrake|redhat|gentoo|suse|slackware>" "If specified, tweaks some scriptlets to be more integrated with the given distribution."
+    AddConfigHelp "Include <filename>" "Read configuration directives from the given file."
+    AddConfigHelp "XDisplay <display location>" "Specifies where scriptlets that use the X server should find one. (Default: :0)"
 }
 
 EnsureHaveRoot() {
@@ -569,7 +667,7 @@ EnsureHaveRoot() {
 # it easy to decide whether or not to pipe its output to $LOGPIPE or not.
 DoWork() {
     # Trap Ctrl+C
-    trap ctrlc_handler INT
+    trap ctrlc_handler INT HUP
 
     # Do everything we need to do to suspend. If anything fails, we don't
     # suspend.  Suspend itself should be the last one in the sequence.
@@ -605,7 +703,7 @@ DoWork() {
 	    break
 	fi
 	if [ -n "$SUSPEND_ABORT" ] ; then
-	    vecho 0 "$EXE: Aborted suspend with Ctrl+C."
+	    vecho 0 "$EXE: Suspend aborted by user."
 	    EXIT_CODE=3
 	    break
 	fi
@@ -634,6 +732,7 @@ ctrlc_handler() {
 VERBOSITY=0
 LOG_VERBOSITY=1
 LOGPIPE="cat"
+ERROR_TEXT=""
 
 EnsureHavePrerequisites
 EnsureHaveRoot
@@ -653,7 +752,7 @@ fi
 DISABLE_HELP=1
 
 LoadScriptlets
-ReadConfigFile
+ReadConfigFile "${CONFIG_FILE}"
 ParseOptions "$@"
 
 # Set a logfile if we need one.
@@ -676,12 +775,11 @@ if [ "$LOGPIPE" = "cat" ] ; then
 else
     # Sigh. Our portable way to obtain exit codes has issues in bash, so if
     # we're using bash, we do it the not-so-portable way :)
-    if shopt > /dev/null 2>&1 ; then
-	# we're using something like bash hopefully! :)
+    if [ -n "$USING_BASH" ] ; then
 	DoWork | $LOGPIPE
 	eval 'EXIT_CODE=${PIPESTATUS[0]}'
     else
-	# Evilness requires to pass the exit code back to us in a pipe.
+	# Evilness required to pass the exit code back to us in a pipe.
 	trap "" INT
 	exec 3>&1
 	eval `
@@ -697,4 +795,4 @@ echo "Resumed at "`date` | $LOGPIPE > /dev/null
 
 exit $EXIT_CODE
 
-# $Id: hibernate.sh 606 2004-11-24 14:13:18Z dagobah $
+# $Id: hibernate.sh 881 2005-04-04 12:31:19Z bernard $
